@@ -47,9 +47,10 @@ gst_implements_interface_supported_default (GstImplementsInterface * iface,
 GType
 gst_implements_interface_get_type (void)
 {
-  static GType gst_interface_type = 0;
+  static volatile gsize gst_interface_type = 0;
 
-  if (!gst_interface_type) {
+  if (g_once_init_enter (&gst_interface_type)) {
+    GType _type;
     static const GTypeInfo gst_interface_info = {
       sizeof (GstImplementsInterfaceClass),
       (GBaseInitFunc) gst_implements_interface_class_init,
@@ -63,10 +64,11 @@ gst_implements_interface_get_type (void)
       NULL
     };
 
-    gst_interface_type = g_type_register_static (G_TYPE_INTERFACE,
+    _type = g_type_register_static (G_TYPE_INTERFACE,
         "GstImplementsInterface", &gst_interface_info, 0);
 
-    g_type_interface_add_prerequisite (gst_interface_type, GST_TYPE_ELEMENT);
+    g_type_interface_add_prerequisite (_type, GST_TYPE_ELEMENT);
+    g_once_init_leave (&gst_interface_type, _type);
   }
 
   return gst_interface_type;
@@ -102,6 +104,8 @@ gst_implements_interface_supported_default (GstImplementsInterface * interface,
 gboolean
 gst_element_implements_interface (GstElement * element, GType iface_type)
 {
+  g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
+
   if (G_TYPE_CHECK_INSTANCE_TYPE (G_OBJECT (element), iface_type)) {
     GstImplementsInterface *iface;
     GstImplementsInterfaceClass *ifclass;
@@ -110,6 +114,11 @@ gst_element_implements_interface (GstElement * element, GType iface_type)
         iface_type, GstImplementsInterface);
     ifclass = GST_IMPLEMENTS_INTERFACE_GET_CLASS (iface);
 
+    /* element implements iface_type but not GstImplementsInterface, so
+     * just assume the other interface is implemented unconditionally */
+    if (ifclass == NULL)
+      return TRUE;
+
     if (ifclass->supported != NULL &&
         ifclass->supported (iface, iface_type) == TRUE) {
       return TRUE;
@@ -127,7 +136,7 @@ gst_element_implements_interface (GstElement * element, GType iface_type)
  * cast a given object to an interface type, and check whether this
  * interface is supported for this specific instance.
  *
- * Returns: a gpointer to the interface type
+ * Returns: (transfer none): a gpointer to the interface type
  */
 
 gpointer
@@ -165,15 +174,11 @@ gst_implements_interface_cast (gpointer from, GType iface_type)
 gboolean
 gst_implements_interface_check (gpointer from, GType type)
 {
-  GstImplementsInterface *iface;
-
   /* check cast, return FALSE if it fails, don't give a warning... */
   if (!G_TYPE_CHECK_INSTANCE_TYPE (from, type)) {
     return FALSE;
   }
 
-  iface = G_TYPE_CHECK_INSTANCE_CAST (from, type, GstImplementsInterface);
-
   /* now, if we're an element (or derivative), is this thing
    * actually implemented for real? */
   if (GST_IS_ELEMENT (from)) {
