@@ -58,7 +58,6 @@ typedef enum {
 #define GST_URI_HANDLER(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_URI_HANDLER, GstURIHandler))
 #define GST_IS_URI_HANDLER(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_URI_HANDLER))
 #define GST_URI_HANDLER_GET_INTERFACE(obj)	(G_TYPE_INSTANCE_GET_INTERFACE ((obj), GST_TYPE_URI_HANDLER, GstURIHandlerInterface))
-#define GST_URI_HANDLER_CLASS(obj)	(G_TYPE_CHECK_CLASS_CAST ((obj), GST_TYPE_URI_HANDLER, GstURIHandler))
 
 /**
  * GstURIHandler:
@@ -76,15 +75,15 @@ typedef struct _GstURIHandlerInterface GstURIHandlerInterface;
  * @get_uri: Method to return the URI currently handled by the element.
  * @set_uri: Method to set a new URI.
  * @get_type_full: Variant of get_type which takes a GType argument. This is 
- *  for use by bindings that need to pass context when creating a URI Handler.  *  If implemented, get_type will be used in preference to get_type_full.
- *      Since: 0.10.15
- * @get_protocols_full: Variant of get_type which takes a GType argument. 
- * This is for use by bindings that need to pass context when creating a URI 
- * Handler. If implemented, get_protocols will be used in preference to 
- * get_protocols_full.
- *      Since: 0.10.15
+ *   for use by bindings that need to pass context when creating a URI Handler.
+ *   If implemented, get_type will be used in preference to get_type_full.
+ *   Since: 0.10.15.
+ * @get_protocols_full: Variant of get_protocols which takes a GType argument.
+ *   This is for use by bindings that need to pass context when creating a URI
+ *   Handler. If implemented, get_protocols will be used in preference to
+ *   get_protocols_full. Since: 0.10.15.
  *
- * #GstElements using this interface should implement these methods.
+ * Any #GstElement using this interface should implement these methods.
  */
 struct _GstURIHandlerInterface {
   GTypeInterface	parent;
@@ -134,6 +133,9 @@ gchar *		gst_uri_get_location		(const gchar * uri);
 gchar *		gst_uri_construct		(const gchar * protocol,
 						 const gchar * location);
 
+gchar *         gst_filename_to_uri             (const gchar * filename,
+                                                 GError     ** error);
+
 GstElement *	gst_element_make_from_uri	(const GstURIType type,
 						 const gchar *    uri,
 						 const gchar *    elementname);
